{% extends "base.html" %}
{% block content %}
<div>
  {{ macros.m_button_group([['globalSettingSaveBtn', '설정 저장']])}}
  {{ macros.m_row_start('5') }}
  {{ macros.m_row_end() }}
  {{ macros.m_hr() }}
  
  <form id='setting' name='setting'>  
    {{ macros.setting_checkbox('use_login', '로그인 사용', value=arg['use_login']) }}
    <div id="use_login_div" class="collapse">
      {{ macros.setting_input_text('web_id', '로그인 ID', value=arg['web_id'], desc=['초기값은 admin입니다.'], col='3') }}
      {{ macros.setting_input_text('web_pw', '로그인 암호', value=arg['web_pw'], col='3', type='password', desc=['초기값은 admin입니다.']) }}
    </div>

    {{ macros.m_hr() }}

    {{ macros.setting_checkbox('use_apikey', 'APIKEY 사용', value=arg['use_apikey'], desc=['On : 모든 API 요청 시 apikey 값을 입력해야 합니다.', '없거나 틀릴 경우 에러코드 403리턴']) }}
    {{ macros.setting_input_text_and_buttons('apikey', 'APIKEY', [['apikey_generate_btn', '자동생성']], col='4', value=arg['apikey']) }}
    {{ macros.m_hr() }}
  </form>
</div> <!--전체-->

<script type="text/javascript">

$(document).ready(function(){
  use_collapse("use_login");
});

$('#use_login').change(function() {
  use_collapse('use_login');
});

$("body").on('click', '#apikey_generate_btn', function(e) {
  e.preventDefault();
  globalSendCommand('apikey_generate', null, null, null, function(ret){
    $("#apikey").val(ret);
  });
});


</script>    
{% endblock %}
